@@ -1,16 +1,15 @@
 <?php
-/* vim: set expandtab sw=4 ts=4 sts=4: */
 /**
  * Single signon for phpMyAdmin
  *
  * This is just example how to use script based single signon with
  * phpMyAdmin, it is not intended to be perfect code and look, only
  * shows how you can integrate this functionality in your application.
- *
- * @package    PhpMyAdmin
- * @subpackage Example
  */
 
+declare(strict_types=1);
+
+// phpcs:disable Squiz.Functions.GlobalFunction
 
 /**
  * This function returns username and password.
@@ -24,14 +23,16 @@
 function get_login_credentials($user)
 {
     /* Optionally we can use passed username */
-    if (!empty($user)) {
-        return array($user, 'password');
+    if (! empty($user)) {
+        return [
+            $user,
+            'password',
+        ];
     }
 
     /* Here we would retrieve the credentials */
-    $credentials = array('root', '');
-
-    return $credentials;
+    return [
+        'root',
+        '',
+    ];
 }
-
-?>
