@@ -13,7 +13,6 @@ use function count;
 use function defined;
 use function error_reporting;
 use function get_class;
-use function headers_sent;
 use function htmlspecialchars;
 use function set_error_handler;
 use function set_exception_handler;
@@ -28,7 +27,6 @@ use const E_ERROR;
 use const E_NOTICE;
 use const E_PARSE;
 use const E_RECOVERABLE_ERROR;
-use const E_STRICT;
 use const E_USER_DEPRECATED;
 use const E_USER_ERROR;
 use const E_USER_NOTICE;
@@ -235,8 +233,13 @@ class ErrorHandler
     {
         $config = $GLOBALS['config'] ?? null;
         $this->hideLocation = ! $config instanceof Config || $config->get('environment') !== 'development';
+        $message = get_class($exception);
+        if (! ($exception instanceof \Error) || ! $this->hideLocation) {
+            $message .= ': ' . $exception->getMessage();
+        }
+
         $this->addError(
-            get_class($exception) . ': ' . $exception->getMessage(),
+            $message,
             (int) $exception->getCode(),
             $exception->getFile(),
             $exception->getLine()
@@ -282,7 +285,7 @@ class ErrorHandler
         }
 
         switch ($error->getNumber()) {
-            case E_STRICT:
+            case 2048: // E_STRICT
             case E_DEPRECATED:
             case E_NOTICE:
             case E_WARNING:
@@ -307,7 +310,7 @@ class ErrorHandler
                 // FATAL error, display it and exit
                 $this->dispFatalError($error);
                 if (! defined('TESTSUITE')) {
-                    exit;
+                    exit; // @codeCoverageIgnore
                 }
         }
     }
@@ -333,12 +336,16 @@ class ErrorHandler
      */
     protected function dispFatalError(Error $error): void
     {
-        if (! headers_sent()) {
-            $this->dispPageStart($error);
+        $response = ResponseRenderer::getInstance();
+        if (! $response->headersSent()) {
+            $response->disable();
+            $response->addHTML('<html><head><title>');
+            $response->addHTML($error->getTitle());
+            $response->addHTML('</title></head>' . "\n");
         }
 
-        echo $error->getDisplay();
-        $this->dispPageEnd();
+        $response->addHTML($error->getDisplay());
+        $response->addHTML('</body></html>');
         if (! defined('TESTSUITE')) {
             exit;
         }
@@ -369,32 +376,6 @@ class ErrorHandler
         return $retval;
     }
 
-    /**
-     * display HTML header
-     *
-     * @param Error $error the error
-     */
-    protected function dispPageStart(?Error $error = null): void
-    {
-        ResponseRenderer::getInstance()->disable();
-        echo '<html><head><title>';
-        if ($error) {
-            echo $error->getTitle();
-        } else {
-            echo 'phpMyAdmin error reporting page';
-        }
-
-        echo '</title></head>';
-    }
-
-    /**
-     * display HTML footer
-     */
-    protected function dispPageEnd(): void
-    {
-        echo '</body></html>';
-    }
-
     /**
      * renders errors not displayed
      */
