@@ -1,11 +1,10 @@
 <?php
-/* vim: set expandtab sw=4 ts=4 sts=4: */
 /**
  * Text Plain Bool2Text Transformations plugin for phpMyAdmin
- *
- * @package    PhpMyAdmin-Transformations
- * @subpackage Bool2Text
  */
+
+declare(strict_types=1);
+
 namespace PhpMyAdmin\Plugins\Transformations\Output;
 
 use PhpMyAdmin\Plugins\Transformations\Abs\Bool2TextTransformationsPlugin;
@@ -14,11 +13,7 @@ use PhpMyAdmin\Plugins\Transformations\Abs\Bool2TextTransformationsPlugin;
  * Handles the Boolean to Text transformation for text plain.
  * Has one option: the output format (default 'T/F')
  * or 'Y/N'
- *
- * @package    PhpMyAdmin-Transformations
- * @subpackage Bool2Text
  */
-// @codingStandardsIgnoreLine
 class Text_Plain_Bool2Text extends Bool2TextTransformationsPlugin
 {
     /**
@@ -28,7 +23,7 @@ class Text_Plain_Bool2Text extends Bool2TextTransformationsPlugin
      */
     public static function getMIMEType()
     {
-        return "Text";
+        return 'Text';
     }
 
     /**
@@ -38,6 +33,6 @@ class Text_Plain_Bool2Text extends Bool2TextTransformationsPlugin
      */
     public static function getMIMESubtype()
     {
-        return "Plain";
+        return 'Plain';
     }
 }
