@@ -14,10 +14,8 @@ class ImportPluginProperties extends PluginPropertyItem
 {
     /**
      * Returns the property item type of either an instance of
-     *  - PhpMyAdmin\Properties\Options\OptionsPropertyOneItem ( f.e. "bool",
-     *  "text", "radio", etc ) or
-     *  - PhpMyAdmin\Properties\Options\OptionsPropertyGroup   ( "root", "main"
-     *  or "subgroup" )
+     *  - PhpMyAdmin\Properties\Options\OptionsPropertyOneItem ( f.e. "bool", "text", "radio", etc ) or
+     *  - PhpMyAdmin\Properties\Options\OptionsPropertyGroup   ( "root", "main" or "subgroup" )
      *  - PhpMyAdmin\Properties\Plugins\PluginPropertyItem     ( "export", "import", "transformations" )
      *
      * @return string
@@ -26,4 +24,9 @@ class ImportPluginProperties extends PluginPropertyItem
     {
         return 'import';
     }
+
+    public function getForceFile(): bool
+    {
+        return true;
+    }
 }
