@@ -7,9 +7,13 @@ declare(strict_types=1);
 
 namespace PhpMyAdmin;
 
+use PhpMyAdmin\Query\Compatibility;
+
+use function __;
+use function _pgettext;
 use function array_diff;
 use function array_merge;
-use function array_push;
+use function array_values;
 use function htmlspecialchars;
 use function in_array;
 use function mb_strtoupper;
@@ -52,10 +56,8 @@ class Types
      * Check whether operator is unary.
      *
      * @param string $op operator name
-     *
-     * @return bool
      */
-    public function isUnaryOperator($op)
+    public function isUnaryOperator($op): bool
     {
         return in_array($op, $this->getUnaryOperators());
     }
@@ -94,9 +96,10 @@ class Types
     public function getTextOperators()
     {
         return [
-            'LIKE',
             'LIKE %...%',
+            'LIKE',
             'NOT LIKE',
+            'NOT LIKE %...%',
             '=',
             '!=',
             'REGEXP',
@@ -128,6 +131,7 @@ class Types
             'LIKE',
             'LIKE %...%',
             'NOT LIKE',
+            'NOT LIKE %...%',
             'IN (...)',
             'NOT IN (...)',
             'BETWEEN',
@@ -135,6 +139,25 @@ class Types
         ];
     }
 
+    /**
+     * UUID search operators
+     *
+     * @return string[]
+     */
+    public function getUUIDOperators()
+    {
+        return [
+            '=',
+            '!=',
+            'LIKE',
+            'LIKE %...%',
+            'NOT LIKE',
+            'NOT LIKE %...%',
+            'IN (...)',
+            'NOT IN (...)',
+        ];
+    }
+
     /**
      * Returns operators for given type
      *
@@ -152,6 +175,8 @@ class Types
             $ret = array_merge($ret, $this->getEnumOperators());
         } elseif ($class === 'CHAR') {
             $ret = array_merge($ret, $this->getTextOperators());
+        } elseif ($class === 'UUID') {
+            $ret = array_merge($ret, $this->getUUIDOperators());
         } else {
             $ret = array_merge($ret, $this->getNumberOperators());
         }
@@ -182,6 +207,7 @@ class Types
             } else {
                 $selected = '';
             }
+
             $html .= '<option value="' . htmlspecialchars($fc) . '"'
                 . $selected . '>'
                 . htmlspecialchars($fc) . '</option>';
@@ -202,200 +228,221 @@ class Types
         $type = mb_strtoupper($type);
         switch ($type) {
             case 'TINYINT':
-                return __(
-                    'A 1-byte integer, signed range is -128 to 127, unsigned range is ' .
-                    '0 to 255'
-                );
+                return __('A 1-byte integer, signed range is -128 to 127, unsigned range is 0 to 255');
+
             case 'SMALLINT':
-                return __(
-                    'A 2-byte integer, signed range is -32,768 to 32,767, unsigned ' .
-                    'range is 0 to 65,535'
-                );
+                return __('A 2-byte integer, signed range is -32,768 to 32,767, unsigned range is 0 to 65,535');
+
             case 'MEDIUMINT':
                 return __(
-                    'A 3-byte integer, signed range is -8,388,608 to 8,388,607, ' .
-                    'unsigned range is 0 to 16,777,215'
+                    'A 3-byte integer, signed range is -8,388,608 to 8,388,607, unsigned range is 0 to 16,777,215'
                 );
+
             case 'INT':
                 return __(
                     'A 4-byte integer, signed range is ' .
                     '-2,147,483,648 to 2,147,483,647, unsigned range is 0 to ' .
                     '4,294,967,295'
                 );
+
             case 'BIGINT':
                 return __(
                     'An 8-byte integer, signed range is -9,223,372,036,854,775,808 ' .
                     'to 9,223,372,036,854,775,807, unsigned range is 0 to ' .
                     '18,446,744,073,709,551,615'
                 );
+
             case 'DECIMAL':
                 return __(
                     'A fixed-point number (M, D) - the maximum number of digits (M) ' .
                     'is 65 (default 10), the maximum number of decimals (D) is 30 ' .
                     '(default 0)'
                 );
+
             case 'FLOAT':
                 return __(
                     'A small floating-point number, allowable values are ' .
                     '-3.402823466E+38 to -1.175494351E-38, 0, and 1.175494351E-38 to ' .
                     '3.402823466E+38'
                 );
+
             case 'DOUBLE':
                 return __(
                     'A double-precision floating-point number, allowable values are ' .
                     '-1.7976931348623157E+308 to -2.2250738585072014E-308, 0, and ' .
                     '2.2250738585072014E-308 to 1.7976931348623157E+308'
                 );
+
             case 'REAL':
-                return __(
-                    'Synonym for DOUBLE (exception: in REAL_AS_FLOAT SQL mode it is ' .
-                    'a synonym for FLOAT)'
-                );
+                return __('Synonym for DOUBLE (exception: in REAL_AS_FLOAT SQL mode it is a synonym for FLOAT)');
+
             case 'BIT':
-                return __(
-                    'A bit-field type (M), storing M of bits per value (default is 1, ' .
-                    'maximum is 64)'
-                );
+                return __('A bit-field type (M), storing M of bits per value (default is 1, maximum is 64)');
+
             case 'BOOLEAN':
                 return __(
-                    'A synonym for TINYINT(1), a value of zero is considered false, ' .
-                    'nonzero values are considered true'
+                    'A synonym for TINYINT(1), a value of zero is considered false, nonzero values are considered true'
                 );
+
             case 'SERIAL':
                 return __('An alias for BIGINT UNSIGNED NOT NULL AUTO_INCREMENT UNIQUE');
+
             case 'DATE':
                 return sprintf(
                     __('A date, supported range is %1$s to %2$s'),
                     '1000-01-01',
                     '9999-12-31'
                 );
+
             case 'DATETIME':
                 return sprintf(
                     __('A date and time combination, supported range is %1$s to %2$s'),
                     '1000-01-01 00:00:00',
                     '9999-12-31 23:59:59'
                 );
+
             case 'TIMESTAMP':
                 return __(
                     'A timestamp, range is 1970-01-01 00:00:01 UTC to 2038-01-09 ' .
                     '03:14:07 UTC, stored as the number of seconds since the epoch ' .
                     '(1970-01-01 00:00:00 UTC)'
                 );
+
             case 'TIME':
                 return sprintf(
                     __('A time, range is %1$s to %2$s'),
                     '-838:59:59',
                     '838:59:59'
                 );
+
             case 'YEAR':
                 return __(
                     'A year in four-digit (4, default) or two-digit (2) format, the ' .
                     'allowable values are 70 (1970) to 69 (2069) or 1901 to 2155 and ' .
                     '0000'
                 );
+
             case 'CHAR':
                 return __(
                     'A fixed-length (0-255, default 1) string that is always ' .
                     'right-padded with spaces to the specified length when stored'
                 );
+
             case 'VARCHAR':
                 return sprintf(
                     __(
-                        'A variable-length (%s) string, the effective maximum length ' .
-                        'is subject to the maximum row size'
+                        'A variable-length (%s) string, the effective maximum length is subject to the maximum row size'
                     ),
                     '0-65,535'
                 );
+
             case 'TINYTEXT':
                 return __(
                     'A TEXT column with a maximum length of 255 (2^8 - 1) characters, ' .
                     'stored with a one-byte prefix indicating the length of the value ' .
                     'in bytes'
                 );
+
             case 'TEXT':
                 return __(
                     'A TEXT column with a maximum length of 65,535 (2^16 - 1) ' .
                     'characters, stored with a two-byte prefix indicating the length ' .
                     'of the value in bytes'
                 );
+
             case 'MEDIUMTEXT':
                 return __(
                     'A TEXT column with a maximum length of 16,777,215 (2^24 - 1) ' .
                     'characters, stored with a three-byte prefix indicating the ' .
                     'length of the value in bytes'
                 );
+
             case 'LONGTEXT':
                 return __(
                     'A TEXT column with a maximum length of 4,294,967,295 or 4GiB ' .
                     '(2^32 - 1) characters, stored with a four-byte prefix indicating ' .
                     'the length of the value in bytes'
                 );
+
             case 'BINARY':
                 return __(
-                    'Similar to the CHAR type, but stores binary byte strings rather ' .
-                    'than non-binary character strings'
+                    'Similar to the CHAR type, but stores binary byte strings rather than non-binary character strings'
                 );
+
             case 'VARBINARY':
                 return __(
                     'Similar to the VARCHAR type, but stores binary byte strings ' .
                     'rather than non-binary character strings'
                 );
+
             case 'TINYBLOB':
                 return __(
                     'A BLOB column with a maximum length of 255 (2^8 - 1) bytes, ' .
                     'stored with a one-byte prefix indicating the length of the value'
                 );
+
             case 'MEDIUMBLOB':
                 return __(
                     'A BLOB column with a maximum length of 16,777,215 (2^24 - 1) ' .
                     'bytes, stored with a three-byte prefix indicating the length of ' .
                     'the value'
                 );
+
             case 'BLOB':
                 return __(
                     'A BLOB column with a maximum length of 65,535 (2^16 - 1) bytes, ' .
                     'stored with a two-byte prefix indicating the length of the value'
                 );
+
             case 'LONGBLOB':
                 return __(
                     'A BLOB column with a maximum length of 4,294,967,295 or 4GiB ' .
                     '(2^32 - 1) bytes, stored with a four-byte prefix indicating the ' .
                     'length of the value'
                 );
+
             case 'ENUM':
                 return __(
-                    'An enumeration, chosen from the list of up to 65,535 values or ' .
-                    "the special '' error value"
+                    'An enumeration, chosen from the list of up to 65,535 values or the special \'\' error value'
                 );
+
             case 'SET':
                 return __('A single value chosen from a set of up to 64 members');
+
             case 'GEOMETRY':
                 return __('A type that can store a geometry of any type');
+
             case 'POINT':
                 return __('A point in 2-dimensional space');
+
             case 'LINESTRING':
                 return __('A curve with linear interpolation between points');
+
             case 'POLYGON':
                 return __('A polygon');
+
             case 'MULTIPOINT':
                 return __('A collection of points');
+
             case 'MULTILINESTRING':
-                return __(
-                    'A collection of curves with linear interpolation between points'
-                );
+                return __('A collection of curves with linear interpolation between points');
+
             case 'MULTIPOLYGON':
                 return __('A collection of polygons');
+
             case 'GEOMETRYCOLLECTION':
                 return __('A collection of geometry objects of any type');
+
             case 'JSON':
-                return __(
-                    'Stores and enables efficient access to data in JSON'
-                    . ' (JavaScript Object Notation) documents'
-                );
+                return __('Stores and enables efficient access to data in JSON (JavaScript Object Notation) documents');
+
             case 'INET6':
                 return __('Intended for storage of IPv6 addresses, as well as IPv4 '
                     . 'addresses assuming conventional mapping of IPv4 addresses '
                     . 'into IPv6 addresses');
+
+            case 'UUID':
+                return __('128-bit UUID (Universally Unique Identifier)');
         }
 
         return '';
@@ -426,12 +473,14 @@ class Types
             case 'BOOLEAN':
             case 'SERIAL':
                 return 'NUMBER';
+
             case 'DATE':
             case 'DATETIME':
             case 'TIMESTAMP':
             case 'TIME':
             case 'YEAR':
                 return 'DATE';
+
             case 'CHAR':
             case 'VARCHAR':
             case 'TINYTEXT':
@@ -448,6 +497,7 @@ class Types
             case 'SET':
             case 'INET6':
                 return 'CHAR';
+
             case 'GEOMETRY':
             case 'POINT':
             case 'LINESTRING':
@@ -457,8 +507,12 @@ class Types
             case 'MULTIPOLYGON':
             case 'GEOMETRYCOLLECTION':
                 return 'SPATIAL';
+
             case 'JSON':
                 return 'JSON';
+
+            case 'UUID':
+                return 'UUID';
         }
 
         return '';
@@ -515,13 +569,12 @@ class Types
                     'VERSION',
                 ];
 
-                if (($isMariaDB && $serverVersion < 100012)
-                    || $serverVersion < 50603
-                ) {
+                if (($isMariaDB && $serverVersion < 100012) || $serverVersion < 50603) {
                     $ret = array_diff($ret, ['INET6_NTOA']);
                 }
 
-                return $ret;
+                return array_values($ret);
+
             case 'DATE':
                 return [
                     'CURRENT_DATE',
@@ -540,6 +593,7 @@ class Types
                     'UTC_TIMESTAMP',
                     'YEAR',
                 ];
+
             case 'NUMBER':
                 $ret = [
                     'ABS',
@@ -595,13 +649,13 @@ class Types
                     'WEEKOFYEAR',
                     'YEARWEEK',
                 ];
-                if (($isMariaDB && $serverVersion < 100012)
-                    || $serverVersion < 50603
-                ) {
+
+                if (($isMariaDB && $serverVersion < 100012) || $serverVersion < 50603) {
                     $ret = array_diff($ret, ['INET6_ATON']);
                 }
 
-                return $ret;
+                return array_values($ret);
+
             case 'SPATIAL':
                 if ($serverVersion >= 50600) {
                     return [
@@ -690,13 +744,21 @@ class Types
      */
     public function getAttributes()
     {
-        return [
+        $serverVersion = $this->dbi->getVersion();
+
+        $attributes = [
             '',
             'BINARY',
             'UNSIGNED',
             'UNSIGNED ZEROFILL',
             'on update CURRENT_TIMESTAMP',
         ];
+
+        if (Compatibility::supportsCompressedColumns($serverVersion)) {
+            $attributes[] = 'COMPRESSED=zlib';
+        }
+
+        return $attributes;
     }
 
     /**
@@ -711,6 +773,7 @@ class Types
     {
         $isMariaDB = $this->dbi->isMariaDB();
         $serverVersion = $this->dbi->getVersion();
+        $isUUIDSupported = Compatibility::isUUIDSupported($this->dbi);
 
         // most used types
         $ret = [
@@ -719,6 +782,11 @@ class Types
             'TEXT',
             'DATE',
         ];
+
+        if ($isUUIDSupported) {
+            $ret[] = 'UUID';
+        }
+
         // numeric
         $ret[_pgettext('numeric types', 'Numeric')] = [
             'TINYINT',
@@ -747,7 +815,7 @@ class Types
         ];
 
         // Text
-        $ret[_pgettext('string types', 'String')] = [
+        $stringTypes = [
             'CHAR',
             'VARCHAR',
             '-',
@@ -767,6 +835,12 @@ class Types
             'ENUM',
             'SET',
         ];
+        if ($isMariaDB && $serverVersion >= 100500) {
+            $stringTypes[] = '-';
+            $stringTypes[] = 'INET6';
+        }
+
+        $ret[_pgettext('string types', 'String')] = $stringTypes;
 
         $ret[_pgettext('spatial types', 'Spatial')] = [
             'GEOMETRY',
@@ -779,19 +853,39 @@ class Types
             'GEOMETRYCOLLECTION',
         ];
 
-        if (($isMariaDB && $serverVersion > 100207)
-            || (! $isMariaDB && $serverVersion >= 50708)
-        ) {
+        if (($isMariaDB && $serverVersion > 100207) || (! $isMariaDB && $serverVersion >= 50708)) {
             $ret['JSON'] = ['JSON'];
         }
 
-        if ($isMariaDB && $serverVersion >= 100500) {
-            array_push($ret[_pgettext('string types', 'String')], '-', 'INET6');
+        if ($isUUIDSupported) {
+            $ret['UUID'] = ['UUID'];
         }
 
         return $ret;
     }
 
+    public function mapAliasToMysqlType(string $alias): string
+    {
+        return [
+            'BOOL' => 'TINYINT',
+            'BOOLEAN' => 'TINYINT',
+            'CHARACTER VARYING' => 'VARCHAR',
+            'FIXED' => 'DECIMAL',
+            'FLOAT4' => 'FLOAT',
+            'FLOAT8' => 'DOUBLE',
+            'INT1' => 'TINYINT',
+            'INT2' => 'SMALLINT',
+            'INT3' => 'MEDIUMINT',
+            'INT4' => 'INT',
+            'INT8' => 'BIGINT',
+            'LONG VARBINARY' => 'MEDIUMBLOB',
+            'LONG VARCHAR' => 'MEDIUMTEXT',
+            'LONG' => 'MEDIUMTEXT',
+            'MIDDLEINT' => 'MEDIUMINT',
+            'NUMERIC' => 'DECIMAL',
+        ][$alias] ?? $alias;
+    }
+
     /**
      * Returns an array of integer types
      *
@@ -818,13 +912,13 @@ class Types
      */
     public function getIntegerRange($type, $signed = true)
     {
-        static $min_max_data = [
+        $min_max_data = [
             'unsigned' => [
-                'tinyint'   => [
+                'tinyint' => [
                     '0',
                     '255',
                 ],
-                'smallint'  => [
+                'smallint' => [
                     '0',
                     '65535',
                 ],
@@ -832,21 +926,21 @@ class Types
                     '0',
                     '16777215',
                 ],
-                'int'       => [
+                'int' => [
                     '0',
                     '4294967295',
                 ],
-                'bigint'    => [
+                'bigint' => [
                     '0',
                     '18446744073709551615',
                 ],
             ],
             'signed' => [
-                'tinyint'   => [
+                'tinyint' => [
                     '-128',
                     '127',
                 ],
-                'smallint'  => [
+                'smallint' => [
                     '-32768',
                     '32767',
                 ],
@@ -854,11 +948,11 @@ class Types
                     '-8388608',
                     '8388607',
                 ],
-                'int'       => [
+                'int' => [
                     '-2147483648',
                     '2147483647',
                 ],
-                'bigint'    => [
+                'bigint' => [
                     '-9223372036854775808',
                     '9223372036854775807',
                 ],
