@@ -9,24 +9,28 @@ namespace PhpMyAdmin;
 
 use PhpMyAdmin\Crypto\Crypto;
 
-use function htmlentities;
+use function base64_decode;
+use function base64_encode;
 use function htmlspecialchars;
 use function http_build_query;
+use function in_array;
 use function ini_get;
 use function is_array;
-use function mb_strpos;
-use function strlen;
-use function in_array;
+use function is_string;
 use function json_encode;
+use function method_exists;
+use function str_contains;
+use function strlen;
 use function strtr;
-use function base64_encode;
-use function base64_decode;
 
 /**
  * Static methods for URL/hidden inputs generating
  */
 class Url
 {
+    /** @var string|null */
+    private static $inputArgSeparator = null;
+
     /**
      * Generates text with hidden inputs.
      *
@@ -40,8 +44,6 @@ class Url
      *                             (can be an array of strings)
      *
      * @return string   string with input fields
-     *
-     * @access public
      */
     public static function getHiddenInputs(
         $db = '',
@@ -49,26 +51,26 @@ class Url
         $indent = 0,
         $skip = []
     ) {
-        global $PMA_Config;
+        global $config;
 
         if (is_array($db)) {
-            $params  =& $db;
+            $params =& $db;
         } else {
             $params = [];
             if (strlen((string) $db) > 0) {
                 $params['db'] = $db;
             }
+
             if (strlen((string) $table) > 0) {
                 $params['table'] = $table;
             }
         }
 
-        if (! empty($GLOBALS['server'])
-            && $GLOBALS['server'] != $GLOBALS['cfg']['ServerDefault']
-        ) {
+        if (! empty($GLOBALS['server']) && $GLOBALS['server'] != $GLOBALS['cfg']['ServerDefault']) {
             $params['server'] = $GLOBALS['server'];
         }
-        if (empty($PMA_Config->getCookie('pma_lang')) && ! empty($GLOBALS['lang'])) {
+
+        if (empty($config->getCookie('pma_lang')) && ! empty($GLOBALS['lang'])) {
             $params['lang'] = $GLOBALS['lang'];
         }
 
@@ -176,8 +178,6 @@ class Url
      * @param bool                          $encrypt whether to encrypt URL params
      *
      * @return string   string with URL parameters
-     *
-     * @access public
      */
     public static function getCommon(array $params = [], $divider = '?', $encrypt = true)
     {
@@ -212,31 +212,29 @@ class Url
      * @param bool                              $encrypt whether to encrypt URL params
      *
      * @return string   string with URL parameters
-     *
-     * @access public
      */
     public static function getCommonRaw(array $params = [], $divider = '?', $encrypt = true)
     {
-        global $PMA_Config;
+        global $config;
 
         // avoid overwriting when creating navigation panel links to servers
-        if (isset($GLOBALS['server'])
+        if (
+            isset($GLOBALS['server'])
             && $GLOBALS['server'] != $GLOBALS['cfg']['ServerDefault']
             && ! isset($params['server'])
-            && ! $PMA_Config->get('is_setup')
+            && ! $config->get('is_setup')
         ) {
             $params['server'] = $GLOBALS['server'];
         }
 
         // Can be null when the user is missing an extension.
-        // See: Core::checkExtensions()
-        if ($PMA_Config !== null && empty($PMA_Config->getCookie('pma_lang')) && ! empty($GLOBALS['lang'])) {
+        if ($config !== null && empty($config->getCookie('pma_lang')) && ! empty($GLOBALS['lang'])) {
             $params['lang'] = $GLOBALS['lang'];
         }
 
         $query = self::buildHttpQuery($params, $encrypt);
 
-        if (($divider !== '?' && $divider !== '&') || strlen($query) > 0) {
+        if (($divider !== '?' && $divider !== self::getArgSeparator()) || strlen($query) > 0) {
             return $divider . $query;
         }
 
@@ -244,18 +242,18 @@ class Url
     }
 
     /**
-     * @param array<string, mixed> $params
-     * @param bool                 $encrypt whether to encrypt URL params
+     * @param array<int|string, mixed> $params
+     * @param bool                     $encrypt whether to encrypt URL params
      *
      * @return string
      */
     public static function buildHttpQuery($params, $encrypt = true)
     {
-        global $PMA_Config;
+        global $config;
 
         $separator = self::getArgSeparator();
 
-        if (! $encrypt || ! $PMA_Config->get('URLQueryEncryption')) {
+        if (! $encrypt || $config === null || ! $config->get('URLQueryEncryption')) {
             return http_build_query($params, '', $separator);
         }
 
@@ -288,30 +286,20 @@ class Url
         }
 
         if ($paramsToEncrypt !== []) {
-            $data['eq'] = self::encryptQuery(json_encode($paramsToEncrypt));
+            $data['eq'] = self::encryptQuery((string) json_encode($paramsToEncrypt));
         }
 
         return http_build_query($data, '', $separator);
     }
 
-    /**
-     * @param string $query
-     *
-     * @return string
-     */
-    public static function encryptQuery($query)
+    public static function encryptQuery(string $query): string
     {
         $crypto = new Crypto();
 
         return strtr(base64_encode($crypto->encrypt($query)), '+/', '-_');
     }
 
-    /**
-     * @param string $query
-     *
-     * @return string|null
-     */
-    public static function decryptQuery($query)
+    public static function decryptQuery(string $query): ?string
     {
         $crypto = new Crypto();
 
@@ -319,55 +307,50 @@ class Url
     }
 
     /**
-     * Returns url separator
+     * Returns url separator character used for separating url parts.
      *
-     * extracted from arg_separator.input as set in php.ini
-     * we do not use arg_separator.output to avoid problems with & and &
+     * Extracted from 'arg_separator.input' as set in php.ini, but prefers '&' and ';'.
      *
-     * @param string $encode whether to encode separator or not,
-     *                       currently 'none' or 'html'
-     *
-     * @return string  character used for separating url parts usually ; or &
-     *
-     * @access public
+     * @see https://www.php.net/manual/en/ini.core.php#ini.arg-separator.input
+     * @see https://www.w3.org/TR/1999/REC-html401-19991224/appendix/notes.html#h-B.2.2
      */
-    public static function getArgSeparator($encode = 'none')
+    public static function getArgSeparator(): string
     {
-        static $separator = null;
-        static $html_separator = null;
-
-        if ($separator === null) {
-            // use separators defined by php, but prefer ';'
-            // as recommended by W3C
-            // (see https://www.w3.org/TR/1999/REC-html401-19991224/appendix
-            // /notes.html#h-B.2.2)
-            $arg_separator = (string) ini_get('arg_separator.input');
-            if (mb_strpos($arg_separator, ';') !== false) {
-                $separator = ';';
-            } elseif (strlen($arg_separator) > 0) {
-                $separator = $arg_separator[0];
-            } else {
-                $separator = '&';
-            }
-            $html_separator = htmlentities($separator);
+        if (is_string(self::$inputArgSeparator)) {
+            return self::$inputArgSeparator;
         }
 
-        switch ($encode) {
-            case 'html':
-                return $html_separator;
-            case 'text':
-            case 'none':
-            default:
-                return $separator;
+        $separator = self::getArgSeparatorValueFromIni();
+        if (! is_string($separator) || $separator === '' || str_contains($separator, '&')) {
+            return self::$inputArgSeparator = '&';
         }
+
+        if (str_contains($separator, ';')) {
+            return self::$inputArgSeparator = ';';
+        }
+
+        // uses first character
+        return self::$inputArgSeparator = $separator[0];
+    }
+
+    /** @return string|false */
+    private static function getArgSeparatorValueFromIni()
+    {
+        /** @psalm-suppress ArgumentTypeCoercion */
+        if (method_exists('PhpMyAdmin\Tests\UrlTest', 'getInputArgSeparator')) {
+            // phpcs:ignore SlevomatCodingStandard.Namespaces.ReferenceUsedNamesOnly.ReferenceViaFullyQualifiedName
+            return \PhpMyAdmin\Tests\UrlTest::getInputArgSeparator();
+        }
+
+        return ini_get('arg_separator.input');
     }
 
     /**
      * @param string $route                Route to use
      * @param array  $additionalParameters Additional URL parameters
      */
-    public static function getFromRoute(string $route, array $additionalParameters = []): string
+    public static function getFromRoute(string $route, array $additionalParameters = [], bool $encrypt = true): string
     {
-        return 'index.php?route=' . $route . self::getCommon($additionalParameters, '&');
+        return 'index.php?route=' . $route . self::getCommon($additionalParameters, self::getArgSeparator(), $encrypt);
     }
 }
